﻿Option Explicit On

Imports System.Runtime
Imports System.Runtime.InteropServices
Imports System.IO
Imports System.Text.ASCIIEncoding

''' <summary>
''' Based on Microsoft code to send raw data directly to a printer.
''' See http://support.microsoft.com/kb/322091, http://support.microsoft.com/kb/154078,
'''     http://social.msdn.microsoft.com/Forums/en-AU/clr/thread/44333478-be4d-4612-a7e1-db8ead3f91ca
''' </summary>
Public Class RawPrinterHelper

	<StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi)> _
	Public Structure DOCINFO
		Dim pDocName As String
		Dim pOutputFile As String
		Dim pDataType As String
	End Structure

	' I think we need OpenPrinterA (and StartDocPrinterA?) because we are using CharSet.Ansi but that is just a guess.
	' The C# version uses OpenPrinterW and CharSet.Unicode.
	<DllImport("winspool.drv", EntryPoint:="OpenPrinterA", SetLastError:=True, CharSet:=CharSet.Ansi, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Shared Function OpenPrinter(ByVal pPrinterName As String, ByRef phPrinter As IntPtr, ByVal pDefault As Integer) As Long
	End Function
	<DllImport("winspool.drv", EntryPoint:="ClosePrinter", SetLastError:=True, CharSet:=CharSet.Auto, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Shared Function ClosePrinter(ByVal hPrinter As IntPtr) As Long
	End Function
	<DllImport("winspool.drv", EntryPoint:="StartDocPrinterA", SetLastError:=True, CharSet:=CharSet.Ansi, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Shared Function StartDocPrinter(ByVal hPrinter As IntPtr, ByVal Level As Integer, ByRef pDocInfo As DOCINFO) As Long
	End Function
	<DllImport("winspool.drv", EntryPoint:="EndDocPrinter", SetLastError:=True, CharSet:=CharSet.Auto, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Shared Function EndDocPrinter(ByVal hPrinter As IntPtr) As Long
	End Function
	<DllImport("winspool.drv", EntryPoint:="StartPagePrinter", SetLastError:=True, CharSet:=CharSet.Auto, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Shared Function StartPagePrinter(ByVal hPrinter As IntPtr) As Long
	End Function
	<DllImport("winspool.drv", EntryPoint:="EndPagePrinter", SetLastError:=True, CharSet:=CharSet.Auto, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Shared Function EndPagePrinter(ByVal hPrinter As IntPtr) As Long
	End Function
	<DllImport("winspool.drv", EntryPoint:="WritePrinter", SetLastError:=True, CharSet:=CharSet.Ansi, ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Shared Function WritePrinter(ByVal hPrinter As IntPtr, ByVal data() As Byte, ByVal buf As Integer, ByRef pcWritten As Integer) As Long
	End Function


	''' <summary>
	''' Sends an unmanaged array of bytes to the specified printer queue.
	''' </summary>
	''' <param name="printerName">The name of the printer.</param>
	''' <param name="byteData">The array of bytes to send to the printer.</param>
	''' <returns>true on success, false on failure.</returns>
	Public Shared Function SendBytesToPrinter(ByVal printerName As String, ByVal byteData() As Byte) As Boolean
		Dim hPrinter As IntPtr = New IntPtr(0) ' The printer handle.
		Dim dwError As Integer = 0			   ' Last error - in case there was trouble.
		Dim di As DOCINFO = New DOCINFO		   ' Describes your document (name, port, data type).
		Dim dwWritten As Integer = 0		   ' The number of bytes written by WritePrinter().
		Dim success As Boolean = False		   ' Your success code.

		' Set up the DOCINFO structure.
		di.pDocName = "My VB RAW Document"
		di.pDataType = "RAW"

		' ReSharper disable EmptyGeneralCatchClause
		Try
			success = OpenPrinter(printerName.Normalize(), hPrinter, IntPtr.Zero)
		Catch ex As Exception
		End Try
		' ReSharper restore EmptyGeneralCatchClause

		If success Then
			If StartDocPrinter(hPrinter, 1, di) Then
				If StartPagePrinter(hPrinter) Then
					' Write your printer-specific bytes to the printer.
					success = WritePrinter(hPrinter, byteData, byteData.Length, dwWritten)
					EndPagePrinter(hPrinter)
				End If
				EndDocPrinter(hPrinter)
			End If
			ClosePrinter(hPrinter)
		End If

		' If you did not succeed, GetLastError may give more information about why not.
		If Not success Then
			dwError = Marshal.GetLastWin32Error()
		End If
		Return success
	End Function

	''' <summary>
	''' Sends the contents of a file to the specified printer queue.
	''' Presumes that the file contains printer-ready data.
	''' Shows how to use the SendBytesToPrinter function.
	''' </summary>
	''' <param name="printerName">The name of the printer.</param>
	''' <param name="fileName">The full path.</param>
	''' <returns>true on success, false on failure.</returns>
	''' 
	Public Shared Function SendFileToPrinter(printerName As String, fileName As String) As Boolean
		' Open the file, and create a BinaryReader.
		Dim success As Boolean = False

		Using fs As FileStream = New FileStream(fileName, FileMode.Open)
			Dim br As BinaryReader = New BinaryReader(fs)
			Dim bytes(fs.Length - 1) As Byte

			' Read the contents of the file into the array.
			bytes = br.ReadBytes(fs.Length)
			' Send the unmanaged bytes to the printer.
			success = SendBytesToPrinter(printerName, bytes)
		End Using

		Return success
	End Function

	''' <summary>
	''' When the function is given a string and a printer name, the function sends the string to the printer as raw bytes.
	''' </summary>
	''' <param name="printerName">The name of the printer.</param>
	''' <param name="text">The string to be sent to the printer.</param>
	Public Shared Function SendStringToPrinter(ByVal printerName As String, ByVal text As String) As Boolean
		Dim success As Boolean
		success = SendBytesToPrinter(printerName, ASCII.GetBytes(text))
		Return success
	End Function

End Class

