﻿Option Explicit On

Imports System.Collections.Generic
Imports System.Text
Imports System.Windows
Imports WPF.Client.PurolatorWebServiceReference
Imports Utilities

Namespace WPF.Client
	Enum SeverityEnum
		''' <summary>
		''' The call was successful without any warnings or error messages.
		''' </summary>
		Success = 0
		''' <summary>
		''' The call was successful, but with warnings.
		''' A warning does not prevent the order from being saved, but may indicate a problem may
		''' need to be addressed.  For example, an order with an invalid email address can be created,
		''' but the system won't attempt to send any status updates to the invalid email address.
		''' </summary>
		Warning = 1
		''' <summary>
		''' The call failed.
		''' An error condition indicates that the Request was invalid in some way and must be
		''' fully corrected before it can be added to OLS.
		''' </summary>
		FatalError = 2
	End Enum

	Class MainWindow
		Private ReadOnly addedPackageIDs As New List(Of String)
		Private lineBreak As String = "---------------------------------------------------------------------" & Environment.NewLine
		Private Const printerName As String = "My ZPL Printer Name"

#Region "Events"

		''' <summary>
		''' Get a randomly generated order, and submit it to OLS using web services.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub OnAddOrderClick(sender As System.Object, e As System.Windows.RoutedEventArgs)
			' Connection information is stored in the App.config file.
			' The Sandbox connection should be used for testing instead of Production to avoid billing for test packages.
			Dim client As ShippingServicesV3SoapClient = New ShippingServicesV3SoapClient("PurolatorWebService")
			' Create a valid order given correct credentials.
			Dim request As OrderRequestV3
			request = ObjectHelper.GetRandomOrderRequest(GetCredentials())

			txtResults.Text += lineBreak
			txtResults.Text += "Sending AddOrder Request." & Environment.NewLine
			' Attempt to add the order to OLS.
			' If there are no Errors, the label for each package will be marked as printed (even if the
			' label code isn't actually send to the printer).
			' If you want to retain maximum flexibility after submitting an order, then set
			' OrderRequest.LabelType to "None"
			' Update the App.config file if you get a CommunicationException indicating the message is too large.
			Dim response As OrderResponseV3 = client.AddOrder(request)

			' Display any errors and display minimal order information.
			DisplayErrorMessages(response)
			DisplayOrderResponseMessage(response, request)
			If response.Status = DirectCast(SeverityEnum.FatalError, Integer) Then
				Return
			End If

			' Store the PackageIDs so we can close the related packages.
			' This application makes no attempt to keep track of packages created in prior sessions.
			For Each package As RequestPackageV3 In request.Packages
				addedPackageIDs.Add(package.PackageID)
			Next


			' ZPL encoded labels can be sent as raw bytes directly to a ZPL printer (http:''www.zebra.com/us/en.html)
			' If OrderRequest.LabelType is IMAGE, then you can (for example) decode it and send it to the printer
			' using a PrintPreviewDialog, or using PrintDocument.
			' (If needed, use PrinterSettings.InstalledPrinters to get a list of installed printers.)

			'' To send each ZPL encoded label to a ZPL printer uncomment this code (and update the printer name).
			'For Each package As ResponsePackage In response.Packages
			'	' This method sends raw ZPL data to the printer directly.
			'	RawPrinterHelper.SendStringToPrinter(printerName, package.ZPLCode)
			'Next
		End Sub

		''' <summary>
		''' Attempt to closeout all shipments created during this session which contain printed labels,
		''' i.e. all packages this application created that were submitted without errors since this application started.
		''' This demo application doesn't attempt to figure out if there are any other packages should be closed out,
		''' and assumes that all packages it successfully creates have had their labels printed.
		''' </summary>
		''' <param name="sender"></param>
		''' <param name="e"></param>
		Private Sub OnCloseoutClick(sender As System.Object, e As System.Windows.RoutedEventArgs)
			' Connection information (the URL) is stored in the App.config file.
			' The Sandbox connection should be used for testing instead of Production to avoid billing for test packages.
			Dim client As ShippingServicesV3SoapClient = New ShippingServicesV3SoapClient("PurolatorWebService")
			' Send array of all packages from this session which haven't yet been closed out.
			Dim request As CloseoutRequest = ObjectHelper.GetCloseoutRequest(GetCredentials(), addedPackageIDs.ToArray())

			txtResults.Text += lineBreak
			txtResults.Text += "Sending Closeout Request." & Environment.NewLine
			Dim response As CloseoutResponse = client.Closeout(request)

			' Display any errors and indicate which packages we close out (or tried to close out).
			DisplayErrorMessages(response)
			DisplayCloseoutResponseMessage(response, request)

			' Clear the package list if no errors occured since we can only close them out once.
			If response.Status <> DirectCast(SeverityEnum.FatalError, Integer) Then
				addedPackageIDs.Clear()
			End If
		End Sub

#End Region

#Region "Helper Methods"

		''' <summary>
		''' Creates a CredentialInfo from the text boxes on the form.
		''' </summary>
		''' <returns>A populated CredentialInfo object.</returns>
		''' <remarks>
		''' The UserName and Password fields must match an active user listed in http://support.pclusa.com/Sandbox/Admin/UserList.aspx
		''' The ClientID must match an active ShipperAccount.  (Purolator support staff typically assigns one ClientID for each facility you ship from.)
		''' </remarks>
		Public Function GetCredentials() As CredentialInfo
			Dim credentials As CredentialInfo = New CredentialInfo With {
			.UserName = txtUsername.Text,
			.Password = txtPassword.Text,
			.ClientID = txtClientID.Text}

			Return credentials
		End Function


		''' <summary>
		''' Append any Errors returned by the web service call to the results text box.
		''' </summary>
		''' <param name="response">The response from the last web service call.</param>
		Public Sub DisplayErrorMessages(response As ResponseBase)
			If response.Messages Is Nothing Then
				Return
			End If

			Dim fullResults As StringBuilder = New StringBuilder
			fullResults.Append(txtResults.Text)

			For Each err As ResponseMessage In response.Messages
				' List the error code for each Error and the Error message text.
				If err.Severity = DirectCast(SeverityEnum.FatalError, Integer) Then
					fullResults.AppendLine(String.Format("Error ({0}): {1}", err.Number, err.Message))
				End If
			Next

			txtResults.Text = fullResults.ToString()
		End Sub

		''' <summary>
		''' Append the results of the AddOrder call to the results text box.
		''' </summary>
		''' <param name="response">The response from the AddOrder webservice call.</param>
		''' <param name="request">The original request passed to the AddOrder webservice call.</param>
		Public Sub DisplayOrderResponseMessage(response As OrderResponseV3, request As OrderRequestV3)
			Dim fullResults As StringBuilder = New StringBuilder()
			fullResults.Append(txtResults.Text)

			If response.Status = DirectCast(SeverityEnum.FatalError, Integer) Then
				fullResults.Append("Attempted to add order: " + request.OrderNumber)
			Else
				fullResults.Append("Added order: " + request.OrderNumber)
			End If

			fullResults.Append(" Package IDs: ")
			For Each package As RequestPackageV3 In request.Packages
				fullResults.Append(package.PackageID)
				fullResults.Append(", ")
			Next
			fullResults.AppendLine()

			txtResults.Text = fullResults.ToString()
		End Sub

		''' <summary>
		''' Append the results of the Closeout call to the results text box.
		''' </summary>
		''' <param name="response">The response from the Closeout webservice call.</param>
		''' <param name="request">The original request passed to the Closeout webservice call.</param>
		Public Sub DisplayCloseoutResponseMessage(response As CloseoutResponse, request As CloseoutRequest)
			Dim fullResults As StringBuilder = New StringBuilder()
			fullResults.Append(txtResults.Text)

			If response.Status = DirectCast(SeverityEnum.FatalError, Integer) Then
				fullResults.Append("Attempted to closeout packages: ")
			Else
				fullResults.Append("Packages closed out: ")
			End If

			For Each packageID As String In request.PackageIDs
				fullResults.Append(packageID)
				fullResults.Append(", ")
			Next
			fullResults.AppendLine()

			txtResults.Text = fullResults.ToString()
		End Sub

#End Region
	End Class
End Namespace
